class Zone(object):

    def __init__(self, id, name):
        self.id = id
        self.name = name

    def __str__(self):
        return 'Zone {id: %d, name: %s}' % (self.id, self.name)


class Record(object):

    def __init__(self, id, name, type, priority, content, ttl):
        self.id = id
        self.name = name
        self.type = type
        self.priority = priority
        self.content = content
        self.ttl = ttl

    def __str__(self):
        return ('Record {'
                'id: %d, '
                'name: %s, '
                'type: %s, '
                'priority: %s, '
                'content: %s, '
                'ttl: %d'
                '}') % (self.id,
                        self.name,
                        self.type,
                        self.priority,
                        self.content,
                        self.ttl)



import json
from .models import Zone
from .models import Record

try:
    from urllib import urlencode
    from urllib2 import Request
    from urllib2 import urlopen
except:
    from urllib.parse import urlencode
    from urllib.request import Request
    from urllib.request import urlopen


class Dozens(object):

    AUTHORIZE_URL = 'http://dozens.jp/api/authorize.json'
    GET_ZONES_URL = 'http://dozens.jp/api/zone.json'
    ADD_ZONE_URL = 'http://dozens.jp/api/zone/create.json'
    DELETE_ZONE_URL = 'http://dozens.jp/api/zone/delete/%s.json'

    GET_RECORDS_URL = 'http://dozens.jp/api/record/%s.json'
    ADD_RECORD_URL = 'http://dozens.jp/api/record/create.json'
    UPDATE_RECORD_URL = 'http://dozens.jp/api/record/update/%d.json'
    DELETE_RECORD_URL = 'http://dozens.jp/api/record/delete/%d.json'

    def __init__(self, user, key):
        self.user = user
        self.key = key

    def start(self):
        headers = {
            'X-auth-user': self.user,
            'X-auth-key': self.key,
            }
        response = self._do_request(self.AUTHORIZE_URL, headers=headers)
        self.token = response.get('auth_token')

    def get_zones(self):
        zones = self.get(self.GET_ZONES_URL)
        if zones:
            return [Zone(int(zone.get('id')), zone.get('name'))
                    for zone in zones.get('domain')]
        return []

    def add_zone(self, name, add_google_apps=False, google_authorize=None):
        params = {'name': name}
        if add_google_apps:
            params['add_google_apps'] = True
            if google_authorize:
                params['google_authorize'] = google_authorize

        zones = self.post(self.ADD_ZONE_URL, params).get('domain')
        for zone in zones:
            if zone.get('name') == name:
                return Zone(zone.get('id'), zone.get('name'))

    def delete_zone(self, zone_id):
        self.delete(self.DELETE_ZONE_URL % zone_id)

    def get_records(self, zone_name):
        records = self.get(self.GET_RECORDS_URL % zone_name)
        if records:
            return [Record(int(record.get('id')),
                           record.get('name'),
                           record.get('type'),
                           record.get('prio'),
                           record.get('content'),
                           int(record.get('ttl')))
                    for record in records.get('record')]
        return []

    def add_record(self,
                   zone_name,
                   record_type,
                   content,
                   record_name=None,
                   priority=None,
                   ttl=7200):
        params = {
            'domain': zone_name,
            'name': record_name,
            'type': record_type,
            'prio': priority,
            'content': content,
            'ttl': ttl,
            }
        records = self.post(self.ADD_RECORD_URL, params).get('record')
        for record in records:
            if record.get('content') == content:
                return Record(int(record.get('id')),
                              record.get('name'),
                              record.get('type'),
                              record.get('prio'),
                              record.get('content'),
                              int(record.get('ttl')))

    def update_record(self, record_id, priority=None, content=None, ttl=None):
        params = {}
        if priority:
            params['prio'] = priority
        if content:
            params['content'] = content
        if ttl:
            params['ttl'] = ttl
        url = self.UPDATE_RECORD_URL % record_id
        records = self.post(url, params).get('record')
        for record in records:
            if int(record.get('id')) == record_id:
                return Record(record_id,
                              record.get('name'),
                              record.get('type'),
                              record.get('prio'),
                              record.get('content'),
                              int(record.get('ttl')))

    def delete_record(self, record_id):
        self.delete(self.DELETE_RECORD_URL % record_id)

    def get(self, url, params={}):
        if params:
            url = url + '?' + urlencode(params)
        return self._do_api_request(url)

    def post(self, url, params={}):
        return self._do_api_request(url, params)

    def delete(self, url, params={}):
        return self._do_api_request(url, params, 'DELETE')

    def _do_api_request(self, url, data={}, method=None):
        headers = {
            'Content-type': 'application/json',
            'X-auth-token': self.token,
            }
        return self._do_request(url, data, headers, method)

    def _do_request(self, url, data={}, headers={}, method=None):
        request = Request(url)
        if method:
            request.get_method = lambda: method
        if data:
            request.add_data(json.dumps(data))
        for key, value in headers.items():
            request.add_header(key, value)
        return json.loads(urlopen(request).read())



import json
import dozens
import mock
from unittest import TestCase

try:
    from urllib2 import addinfourl
    from StringIO import StringIO
except:
    from urllib.response import addinfourl
    from io import StringIO


class DozensTestCase(TestCase):

    def test_start(self):
        with mock.patch('dozens.dozens.urlopen') as m:
            m.return_value = self.mock_response({'auth_token': 'dummy_token'})

            testee = dozens.Dozens('user', 'key')
            testee.start()
            self.assertEqual(testee.token, 'dummy_token')

            request = m.call_args[0][0]
            self.assertEqual(request.get_full_url(), testee.AUTHORIZE_URL)
            self.assertEqual(request.get_header('X-auth-user'), 'user')
            self.assertEqual(request.get_header('X-auth-key'), 'key')
            self.assertEqual(request.get_method(), 'GET')

    def test_get_zones(self):
        testee = dozens.Dozens('user', 'key')
        testee.token = 'token'
        dummy = [
            {'id': 1, 'name': 'name1'},
            {'id': 2, 'name': 'name2'},
            ]
        response = self.mock_response({'domain': dummy})

        zones, request = self.request(response, testee.get_zones)
        self.assertEqual(zones[0].id, 1)
        self.assertEqual(zones[0].name, 'name1')
        self.assertEqual(zones[1].id, 2)
        self.assertEqual(zones[1].name, 'name2')
        self.assertEqual(request.get_full_url(), testee.GET_ZONES_URL)
        self.assertEqual(request.get_method(), 'GET')

    def test_add_zone(self):
        testee = dozens.Dozens('user', 'key')
        testee.token = 'token'
        dummy = [
            {'id': 1, 'name': 'name1'},
            {'id': 2, 'name': 'name2'},
            ]
        response = self.mock_response({'domain': dummy})
        args = ('name2', True, 'google')

        zone, request = self.request(response, testee.add_zone, *args)
        self.assertEqual(zone.id, 2)
        self.assertEqual(zone.name, 'name2')
        self.assertEqual(request.get_full_url(), testee.ADD_ZONE_URL)
        self.assertEqual(request.get_method(), 'POST')

        params = json.loads(request.data)
        self.assertEqual(params['name'], 'name2')
        self.assertEqual(params['google_authorize'], 'google')
        self.assertTrue(params['add_google_apps'])

    def test_delete_zone(self):
        testee = dozens.Dozens('user', 'key')
        testee.token = 'token'
        response = self.mock_response({})

        result, request = self.request(response, testee.delete_zone, 1)
        self.assertEqual(request.get_full_url(), testee.DELETE_ZONE_URL % 1)
        self.assertEqual(request.get_method(), 'DELETE')

    def test_get_records(self):
        dummy = [
            {'id': 1,
             'name': 'name1',
             'type': 'type1',
             'prio': 'prio1',
             'content': 'content1',
             'ttl': 7200},
            {'id': 2,
             'name': 'name2',
             'type': 'type2',
             'prio': 'prio2',
             'content': 'content2',
             'ttl': 7200},
            ]
        response = self.mock_response({'record': dummy})
        testee = dozens.Dozens('user', 'key')
        testee.token = 'token'

        records, request = self.request(response, testee.get_records, 'domain')
        self.assertEqual(records[0].id, 1)
        self.assertEqual(records[0].name, 'name1')
        self.assertEqual(records[0].type, 'type1')
        self.assertEqual(records[0].priority, 'prio1')
        self.assertEqual(records[0].content, 'content1')
        self.assertEqual(records[0].ttl, 7200)

        self.assertEqual(records[1].id, 2)
        self.assertEqual(records[1].name, 'name2')
        self.assertEqual(records[1].type, 'type2')
        self.assertEqual(records[1].priority, 'prio2')
        self.assertEqual(records[1].content, 'content2')
        self.assertEqual(records[1].ttl, 7200)

        self.assertEqual(request.get_full_url(),
                         testee.GET_RECORDS_URL % 'domain')
        self.assertEqual(request.get_method(), 'GET')

    def test_add_record(self):
        dummy = [
            {'id': 1,
             'name': 'name1',
             'type': 'type1',
             'prio': 'prio1',
             'content': 'content1',
             'ttl': 7200},
            {'id': 2,
             'name': 'name2',
             'type': 'type2',
             'prio': 'prio2',
             'content': 'content2',
             'ttl': 7200},
            ]
        response = self.mock_response({'record': dummy})
        testee = dozens.Dozens('user', 'key')
        testee.token = 'token'

        args = ('domain', 'type1', 'content1', 'name1', 'prio1', 7200)
        record, request = self.request(response, testee.add_record, *args)
        self.assertEqual(record.id, 1)
        self.assertEqual(record.name, 'name1')
        self.assertEqual(record.type, 'type1')
        self.assertEqual(record.priority, 'prio1')
        self.assertEqual(record.content, 'content1')
        self.assertEqual(record.ttl, 7200)
        self.assertEqual(request.get_full_url(), testee.ADD_RECORD_URL)
        self.assertEqual(request.get_method(), 'POST')

        params = json.loads(request.data)
        self.assertEqual(params['domain'], 'domain')
        self.assertEqual(params['type'], 'type1')
        self.assertEqual(params['content'], 'content1')
        self.assertEqual(params['name'], 'name1')
        self.assertEqual(params['prio'], 'prio1')
        self.assertEqual(params['ttl'], 7200)

    def test_update_record(self):
        dummy = [
            {'id': 1,
             'name': 'name1',
             'type': 'type1',
             'prio': 'prio1',
             'content': 'content1',
             'ttl': 7200},
            {'id': 2,
             'name': 'name2',
             'type': 'type2',
             'prio': 'prio2',
             'content': 'content2',
             'ttl': 7200},
            ]
        response = self.mock_response({'record': dummy})
        testee = dozens.Dozens('user', 'key')
        testee.token = 'token'

        args = (1, 'prio1', 'content1', 7200)
        record, request = self.request(response, testee.update_record, *args)
        self.assertEqual(record.id, 1)
        self.assertEqual(record.name, 'name1')
        self.assertEqual(record.type, 'type1')
        self.assertEqual(record.priority, 'prio1')
        self.assertEqual(record.content, 'content1')
        self.assertEqual(record.ttl, 7200)
        self.assertEqual(request.get_full_url(), testee.UPDATE_RECORD_URL % 1)
        self.assertEqual(request.get_method(), 'POST')

        params = json.loads(request.data)
        self.assertEqual(params['prio'], 'prio1')
        self.assertEqual(params['content'], 'content1')
        self.assertEqual(params['ttl'], 7200)

    def test_delete_record(self):
        testee = dozens.Dozens('user', 'key')
        testee.token = 'token'
        response = self.mock_response({})

        result, request = self.request(response, testee.delete_record, 1)
        self.assertEqual(request.get_full_url(), testee.DELETE_RECORD_URL % 1)
        self.assertEqual(request.get_method(), 'DELETE')

    def test_get_empty_zones(self):
        testee = dozens.Dozens('user', 'key')
        testee.token = 'token'
        response = self.mock_response([])

        zones, request = self.request(response, testee.get_zones)
        self.assertEqual(zones, [])
        self.assertEqual(request.get_full_url(), testee.GET_ZONES_URL)
        self.assertEqual(request.get_method(), 'GET')

    def test_get_empty_records(self):
        testee = dozens.Dozens('user', 'key')
        testee.token = 'token'
        response = self.mock_response([])

        records, request = self.request(response, testee.get_records, 'domain')
        self.assertEqual(records, [])
        self.assertEqual(request.get_full_url(),
                         testee.GET_RECORDS_URL % 'domain')
        self.assertEqual(request.get_method(), 'GET')

    def test_get_with_query(self):
        with mock.patch('dozens.dozens.urlopen') as m:
            m.return_value = self.mock_response({})

            testee = dozens.Dozens('user', 'key')
            testee.token = 'token'
            testee.get('http://test.com/url', {'key': 'value'})

            request = m.call_args[0][0]
            self.assertEqual(request.get_full_url(), 'http://test.com/url?key=value')
            self.assertEqual(request.get_method(), 'GET')

    def test_models(self):
        zone = dozens.Zone(1, 'name')
        record = dozens.Record(1, 'name', 'type', 'priority', 'content', 7200)
        self.assertEqual(str(zone), 'Zone {id: 1, name: name}')
        self.assertEqual(str(record), ('Record {'
                                       'id: 1, '
                                       'name: name, '
                                       'type: type, '
                                       'priority: priority, '
                                       'content: content, '
                                       'ttl: 7200'
                                       '}'))

    def request(self, response, method, *args):
        with mock.patch('dozens.dozens.urlopen') as m:
            m.return_value = response
            result = method(*args)

            request = m.call_args[0][0]
            content_type = request.get_header('Content-type')
            token = request.get_header('X-auth-token')

            self.assertEqual(content_type, 'application/json')
            self.assertEqual(token, 'token')
            return result, request

    def mock_response(self, value):
        body = StringIO(json.dumps(value))
        response = addinfourl(body, {}, 'dummy_url')
        response.code = 200
        return response



import os
from setuptools import setup
from setuptools import find_packages

version = '0.0.5'
classifiers = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python',
]

try:
    here = os.path.abspath(os.path.dirname(__file__))
    README = open(os.path.join(here, 'README.rst')).read()
except IOError:
    README = ''

setup(
    name='dozens',
    version=version,
    packages=find_packages(),
    test_suite='tests',
    tests_require=['mock'],

    author='mikamix',
    author_email='tmp@mikamix.net',
    description='Wrapper for Dozens API',
    long_description=README,
    keywords=['dozens'],
    classifiers=classifiers,
    license='MIT',
)
